CON

  #0, _Inactive, _Active, _Deactivate
  {The above does the same as the three folling lines.
  _Inactive = 0
  _Active = 1
  _Deactivate = 2 }
  
  #0, _Off, _On

  _HighestLedPin = 31 ' 
  
VAR

  long cog, ms, stack[32]
  long time[_HighestLedPin + 1], onInterval[_HighestLedPin + 1], offInterval[_HighestLedPin + 1], count[_HighestLedPin + 1]
  byte state[_HighestLedPin + 1], active[_HighestLedPin + 1]
  
PUB Start
'' The Start method must be called before using the
'' methods AddLed or OffLed.

  ms := clkfreq / 1000

  Stop
  cog := cognew(LedFlash, @stack) + 1
  
  result := cog
     
PUB Stop 

  repeat result from 0 to _HighestLedPin
    if active[result] == _Active
      OffLed(result)
      
  waitcnt(clkfreq / 50 + cnt) ' give loop time to deactivate all LEDs
      
  if cog
    cogstop(cog~ - 1)

PUB AddLed(pin, onMs_, offMs_, count_)
'' onMs_ is the time the LED is to be on each flash.
'' onMs_ is limited to 26,843ms (over 26 seconds).
'' offMs_ is the time LED will be off between flashes.
'' offMs_ has the same limit as onMs_.
'' count_ is the number of times to flash the LED.
'' If count_ is set to zero then the LED will continue
'' to flash indefinitely.
'' The Start method must be called prior to calling this
'' method.
'' This method returns the active state
'' of the pin prior to calling this method or a -1 if the
'' pin number was out of range.

  if pin < 0 or pin > _HighestLedPin
    return -1      ' return a -1 if pin number is out of range

  result := active[pin]
  active[pin] := _Inactive ' To stop the LED cog from changing the
                           ' state of he pin while we set it here.

  count[pin] := 0 #> count_ <# POSX  ' limit count to positive numbers.
 
  onInterval[pin] := 0 #> onMs_ * ms <# POSX    ' limit intervals to positive numbers.
  offInterval[pin] := 0 #> offMs_ * ms <# POSX
  if count[pin] == 0
    count[pin] := -1
  state[pin] := _Off
  time[pin] := cnt
  active[pin] := _Active
  
PUB OffLed(pin)
'' Used to turn off a flashing LED at "pin".
'' The pin will be set as an input.
'' This method returns the active state
'' of the pin.

  result := active[pin]
  active[pin] := _Deactivate
  
PRI LedFlash
'' Runs in its own cog.

  repeat
    repeat result from 0 to _HighestLedPin  ' check each pin
      if active[result] == _Deactivate  ' should pin be deativated?
        outa[result] := 0
        dira[result] := 0
      if active[result] == _Active  ' is pin active?
        dira[result] := 1
        if state[result] == _Off
          
          if cnt - time[result] > offInterval[result]  ' has pin been off long enough?
            time[result] += offInterval[result]
            state[result] := _On
            outa[result] := 1            
        else
          if cnt - time[result] > onInterval[result]  ' has pin been on long enough?          
            time[result] += onInterval[result]
            state[result] := _Off
            outa[result] := 0
            if count[result] <> -1
              if --count[result] =< 0                         ' decrement count if not -1
                active[result] := _Inactive           
                dira[result] := 0 
     